@@ -3,6 +3,7 @@ import sys
 import logging
 import tempfile
 import os
+import asyncio
 from pytest import mark, fixture, raises
 
 pytestmark = [mark.integration, mark.jobs]
@@ -72,10 +73,7 @@ class JobLifeCycleTests:
     async def test_copy_job(self, job):
         job_copy = await job.copy("CLI Automated Job Copy")
         assert job_copy.id != job.id
-        await job_copy.wait_for("stopped", 120)
-        assert job_copy.status == "stopped"
-        await job_copy.start()
-        await job_copy.wait_for("running", 60)
+        await job_copy.wait_for("running", 120)
         assert job_copy.status == "running"
         await job_copy.stop()
         await job_copy.wait_for("stopped", 60)
@@ -102,14 +100,14 @@ class JobLifeCycleTests:
             ),
         )
         assert training_job.id
-        training_job = await training_job.wait_for("stopped", 180)
-        assert training_job.status == "stopped"
+        training_job = await training_job.wait_for("finished", 180)
+        assert training_job.status == "finished"
         await training_job.remove()
 
     async def test_remove_job(self, job):
         assert job.status == "running"
         await job.stop()
-        await job.wait_for("stopped", 60)
+        await job.wait_for("stopped", 90)
         await job.remove()
         job = await job.wait_for("archived", 60)
         assert job is None
@@ -140,15 +138,16 @@ class JobLifeCycleTests:
             model=dict(git_uri="git@github.com:trainML/test-private.git"),
         )
         await job.wait_for("running")
-        await job.connect()
-        await job.attach()
+        attach_task = asyncio.create_task(job.attach())
+        connect_task = asyncio.create_task(job.connect())
+        await asyncio.gather(attach_task, connect_task)
         await job.refresh()
-        assert job.status == "stopped"
+        assert job.status == "finished"
         await job.disconnect()
         await job.remove()
         upload_contents = os.listdir(temp_dir.name)
         result = any(
-            "CLI_Automated_MXNet_Test_1" in content
+            "CLI_Automated_Tensorflow_Test_1" in content
             for content in upload_contents
         )
         assert result is not None
