@@ -6,7 +6,12 @@ from pytest import mark, fixture, raises
 from aiohttp import WSMessage, WSMsgType
 
 import trainml.jobs as specimen
-from trainml.exceptions import ApiError, JobError
+from trainml.exceptions import (
+    ApiError,
+    JobError,
+    SpecificationError,
+    TrainMLException,
+)
 
 pytestmark = [mark.unit, mark.jobs]
 
@@ -158,7 +163,7 @@ class CleanDatasetSelectionTests:
     def test_existing_dataset_by_name_not_found(
         self, mock_my_datasets, mock_public_datasets
     ):
-        with raises(ValueError):
+        with raises(SpecificationError):
             specimen._clean_datasets_selection(
                 [dict(name="Not Found", type="existing")],
                 "trainml",
@@ -169,7 +174,7 @@ class CleanDatasetSelectionTests:
     def test_public_dataset_by_name_not_found(
         self, mock_my_datasets, mock_public_datasets
     ):
-        with raises(ValueError):
+        with raises(SpecificationError):
             specimen._clean_datasets_selection(
                 [dict(name="Not Found", type="public")],
                 "trainml",
@@ -180,7 +185,7 @@ class CleanDatasetSelectionTests:
     def test_invalid_dataset_type_specified(
         self, mock_my_datasets, mock_public_datasets
     ):
-        with raises(ValueError):
+        with raises(SpecificationError):
             specimen._clean_datasets_selection(
                 [dict(name="Not Found", type="invalid")],
                 "trainml",
@@ -191,7 +196,7 @@ class CleanDatasetSelectionTests:
     def test_invalid_dataset_identifier_specified(
         self, mock_my_datasets, mock_public_datasets
     ):
-        with raises(ValueError):
+        with raises(SpecificationError):
             specimen._clean_datasets_selection(
                 [dict(dataset_id="Not Found", type="invalid")],
                 "trainml",
@@ -256,7 +261,7 @@ class JobsTests:
             ),
             worker_count=1,
             worker_commands=[],
-            environment=dict(type="DEEPLEARNING_PY37"),
+            environment=dict(type="DEEPLEARNING_PY38"),
             data=dict(datasets=[]),
             model=dict(),
             vpn=dict(net_prefix_type_id=1),
@@ -316,7 +321,7 @@ class JobsTests:
             gpu_count=1,
             disk_size=10,
         )
-        with raises(ValueError):
+        with raises(SpecificationError):
             await jobs.create(**requested_config)
 
 
@@ -441,68 +446,17 @@ class JobTests:
             autospec=True,
         ) as mock_connection:
             connection = mock_connection.return_value
-            connection.status = "stopped"
+            connection.status = "removed"
             resp = await job.disconnect()
             connection.stop.assert_called_once()
-            assert resp == "stopped"
+            assert resp == "removed"
 
     @mark.asyncio
     async def test_job_remove(self, job, mock_trainml):
-        with patch(
-            "trainml.jobs.Connection",
-            autospec=True,
-        ) as mock_connection:
-            connection = mock_connection.return_value
-            api_response = dict()
-            mock_trainml._query = AsyncMock(return_value=api_response)
-            await job.remove()
-            mock_trainml._query.assert_called_once_with(
-                "/job/job-id-1", "DELETE"
-            )
-            connection.remove.assert_not_called()
-
-    @mark.asyncio
-    async def test_job_remove_with_connection(self, mock_trainml):
-        with patch(
-            "trainml.jobs.Connection",
-            autospec=True,
-        ) as mock_connection:
-            connection = mock_connection.return_value
-            job = specimen.Job(
-                mock_trainml,
-                **{
-                    "customer_uuid": "cus-id-1",
-                    "job_uuid": "job-id-1",
-                    "name": "test notebook",
-                    "type": "interactive",
-                    "status": "new",
-                    "provider": "trainml",
-                    "data": {
-                        "datasets": [],
-                        "output_type": "local",
-                        "output_uri": "~/tensorflow-example/output",
-                        "status": "ready",
-                    },
-                    "vpn": {
-                        "status": "new",
-                        "cidr": "10.106.171.0/24",
-                        "client": {
-                            "port": "36017",
-                            "id": "cus-id-1",
-                            "address": "10.106.171.253",
-                            "ssh_port": 46600,
-                        },
-                        "net_prefix_type_id": 1,
-                    },
-                },
-            )
-            api_response = dict()
-            mock_trainml._query = AsyncMock(return_value=api_response)
-            await job.remove()
-            mock_trainml._query.assert_called_once_with(
-                "/job/job-id-1", "DELETE"
-            )
-            connection.remove.assert_called_once()
+        api_response = dict()
+        mock_trainml._query = AsyncMock(return_value=api_response)
+        await job.remove()
+        mock_trainml._query.assert_called_once_with("/job/job-id-1", "DELETE")
 
     @mark.asyncio
     async def test_job_refresh(self, job, mock_trainml):
@@ -519,7 +473,7 @@ class JobTests:
         assert job.status == "running"
         assert response.status == "running"
 
-    def test_job_ws_msg_handler(self, job, capsys):
+    def test_job_default_ws_msg_handler(self, job, capsys):
         msg = WSMessage(
             type=WSMsgType.TEXT,
             data=json.dumps(
@@ -533,12 +487,14 @@ class JobTests:
             ),
             extra=None,
         )
-        handler = job._get_msg_handler()
+        handler = job._get_msg_handler(None)
         handler(msg)
         captured = capsys.readouterr()
         assert captured.out == "02/11/2021, 15:35:45: Epoch (1/1000)\n"
 
-    def test_job_ws_msg_handler_multiple_workers(self, mock_trainml, capsys):
+    def test_job_default_ws_msg_handler_multiple_workers(
+        self, mock_trainml, capsys
+    ):
         job = specimen.Job(
             mock_trainml,
             **{
@@ -576,13 +532,35 @@ class JobTests:
             ),
             extra=None,
         )
-        handler = job._get_msg_handler()
+        handler = job._get_msg_handler(None)
         handler(msg)
         captured = capsys.readouterr()
         assert (
             captured.out == "02/11/2021, 15:35:45: Worker 1 - Epoch (1/1000)\n"
         )
 
+    def test_job_custom_ws_msg_handler(self, job, capsys):
+        def custom_handler(msg):
+            print(msg.get("stream"))
+
+        msg = WSMessage(
+            type=WSMsgType.TEXT,
+            data=json.dumps(
+                {
+                    "msg": "Epoch (1/1000)\n",
+                    "time": 1613079345318,
+                    "type": "subscription",
+                    "stream": "worker-id-1",
+                    "job_worker_uuid": "worker-id-1",
+                }
+            ),
+            extra=None,
+        )
+        handler = job._get_msg_handler(custom_handler)
+        handler(msg)
+        captured = capsys.readouterr()
+        assert captured.out == "worker-id-1\n"
+
     @mark.asyncio
     async def test_job_attach(self, job, mock_trainml):
         api_response = None
@@ -645,7 +623,7 @@ class JobTests:
             },
         )
         mock_trainml.jobs.create = AsyncMock()
-        with raises(TypeError):
+        with raises(SpecificationError):
             await job.copy("new job")
         mock_trainml.jobs.create.assert_not_called()
 
@@ -685,7 +663,7 @@ class JobTests:
     async def test_job_wait_for_incorrect_status(self, job, mock_trainml):
         api_response = None
         mock_trainml._query = AsyncMock(return_value=api_response)
-        with raises(ValueError):
+        with raises(SpecificationError):
             await job.wait_for("ready")
         mock_trainml._query.assert_not_called()
 
@@ -725,7 +703,7 @@ class JobTests:
             "status": "new",
         }
         mock_trainml._query = AsyncMock(return_value=api_response)
-        with raises(TimeoutError):
+        with raises(TrainMLException):
             await job.wait_for("running", 10)
         mock_trainml._query.assert_called()
 
