@@ -4,7 +4,12 @@ import math
 import asyncio
 from datetime import datetime
 
-from .exceptions import DatasetError, ApiError
+from .exceptions import (
+    DatasetError,
+    ApiError,
+    SpecificationError,
+    TrainMLException,
+)
 from .connections import Connection
 
 
@@ -22,15 +27,16 @@ class Datasets(object):
         return datasets
 
     async def list_public(self):
-        resp = await self.trainml._query(f"/dataset/public", "GET")
+        resp = await self.trainml._query(f"/dataset/pub/public", "GET")
         datasets = [Dataset(self.trainml, **dataset) for dataset in resp]
         return datasets
 
     async def create(self, name, source_type, source_uri, **kwargs):
         if kwargs.get("provider") and kwargs.get("provider") != "trainml":
             if not kwargs.get("disk_size"):
-                raise AttributeError(
-                    "'disk_size' attribute required for non-trainML providers"
+                raise SpecificationError(
+                    "disk_size",
+                    "'disk_size' attribute required for non-trainML providers",
                 )
         data = dict(
             name=name,
@@ -122,23 +128,31 @@ class Dataset:
         connection = Connection(
             self.trainml, entity_type="dataset", id=self.id, entity=self
         )
-        await connection.remove()  # dataset connections are only used once
+        await connection.stop()
         return connection.status
 
     async def remove(self):
         await self.trainml._query(f"/dataset/pub/{self._id}", "DELETE")
 
-    def _ws_msg_handler(self, msg):
-        data = json.loads(msg.data)
-        if data.get("type") == "subscription":
-            timestamp = datetime.fromtimestamp(int(data.get("time")) / 1000)
-            print(
-                f"{timestamp.strftime('%m/%d/%Y, %H:%M:%S')}: {data.get('msg').rstrip()}"
-            )
+    def _get_msg_handler(self, msg_handler):
+        def handler(msg):
+            data = json.loads(msg.data)
+            if data.get("type") == "subscription":
+                if msg_handler:
+                    msg_handler(data)
+                else:
+                    timestamp = datetime.fromtimestamp(
+                        int(data.get("time")) / 1000
+                    )
+                    print(
+                        f"{timestamp.strftime('%m/%d/%Y, %H:%M:%S')}: {data.get('msg').rstrip()}"
+                    )
 
-    async def attach(self):
+        return handler
+
+    async def attach(self, msg_handler=None):
         await self.trainml._ws_subscribe(
-            "dataset", self.id, self._ws_msg_handler
+            "dataset", self.id, self._get_msg_handler(msg_handler)
         )
 
     async def refresh(self):
@@ -149,8 +163,9 @@ class Dataset:
     async def wait_for(self, status, timeout=300):
         valid_statuses = ["ready", "archived"]
         if not status in valid_statuses:
-            raise ValueError(
-                f"Invalid wait_for status {status}.  Valid statuses are: {valid_statuses}"
+            raise SpecificationError(
+                "status",
+                f"Invalid wait_for status {status}.  Valid statuses are: {valid_statuses}",
             )
         if self.status == status:
             return
@@ -173,4 +188,4 @@ class Dataset:
                 count += 1
                 logging.debug(f"self: {self}, retry count {count}")
 
-        raise TimeoutError(f"Timeout waiting for {status}")
+        raise TrainMLException(f"Timeout waiting for {status}")
