@@ -2,9 +2,15 @@ import json
 import asyncio
 import math
 import logging
+import warnings
 from datetime import datetime
 
-from trainml.exceptions import ApiError, JobError
+from trainml.exceptions import (
+    ApiError,
+    JobError,
+    SpecificationError,
+    TrainMLException,
+)
 from trainml.connections import Connection
 
 
@@ -34,7 +40,9 @@ def _clean_datasets_selection(
                     None,
                 )
                 if not selected_dataset:
-                    raise ValueError(f"Dataset {dataset} Not Found")
+                    raise SpecificationError(
+                        "datasets", f"Dataset {dataset} Not Found"
+                    )
                 datasets.append(
                     dict(
                         dataset_uuid=selected_dataset.id,
@@ -52,7 +60,9 @@ def _clean_datasets_selection(
                     None,
                 )
                 if not selected_dataset:
-                    raise ValueError(f"Dataset {dataset} Not Found")
+                    raise SpecificationError(
+                        "datasets", f"Dataset {dataset} Not Found"
+                    )
                 datasets.append(
                     dict(
                         dataset_uuid=selected_dataset.id,
@@ -60,12 +70,14 @@ def _clean_datasets_selection(
                     )
                 )
             else:
-                raise ValueError(
-                    "Invalid dataset specification, 'type' must be in ['existing','public']"
+                raise SpecificationError(
+                    "datasets",
+                    "Invalid dataset specification, 'type' must be in ['existing','public']",
                 )
         else:
-            raise ValueError(
-                "Invalid dataset specification, either 'id' or 'name' must be provided"
+            raise SpecificationError(
+                "datasets",
+                "Invalid dataset specification, either 'id' or 'name' must be provided",
             )
     return datasets
 
@@ -92,12 +104,19 @@ class Jobs(object):
         disk_size,
         worker_count=1,
         worker_commands=[],
-        environment=dict(type="DEEPLEARNING_PY37"),
+        environment=dict(type="DEEPLEARNING_PY38"),
         data=dict(datasets=[]),
         model=dict(),
         vpn=dict(net_prefix_type_id=1),
         **kwargs,
     ):
+
+        if type in ["headless", "interactive"]:
+            new_type = "notebook" if type == "interactive" else "training"
+            warnings.warn(
+                f"'{type}' type is deprecated, use '{new_type}' instead.",
+                DeprecationWarning,
+            )
         gpu_type_task = asyncio.create_task(self.trainml.gpu_types.list())
         my_datasets_task = asyncio.create_task(self.trainml.datasets.list())
         public_datasets_task = asyncio.create_task(
@@ -113,7 +132,7 @@ class Jobs(object):
             None,
         )
         if not selected_gpu_type:
-            raise ValueError("GPU Type Not Found")
+            raise SpecificationError("gpu_type", "GPU Type Not Found")
 
         if data:
             datasets = _clean_datasets_selection(
@@ -243,11 +262,6 @@ class Job:
         return connection.status
 
     async def remove(self):
-        if self._job.get("data").get("output_type") == "local":
-            connection = Connection(
-                self.trainml, entity_type="job", id=self.id, entity=self
-            )
-            await connection.remove()
         await self.trainml._query(f"/job/{self._id}", "DELETE")
 
     async def refresh(self):
@@ -255,38 +269,44 @@ class Job:
         self.__init__(self.trainml, **resp)
         return self
 
-    def _get_msg_handler(self):
+    def _get_msg_handler(self, msg_handler):
         worker_numbers = {
             w.get("job_worker_uuid"): ind + 1
             for ind, w in enumerate(self._workers)
         }
 
-        def msg_handler(msg):
+        def handler(msg):
             data = json.loads(msg.data)
             if data.get("type") == "subscription":
-                timestamp = datetime.fromtimestamp(
-                    int(data.get("time")) / 1000
-                )
-                if len(self._workers) > 1:
-                    print(
-                        f"{timestamp.strftime('%m/%d/%Y, %H:%M:%S')}: Worker {worker_numbers.get(data.get('stream'))} - {data.get('msg').rstrip()}"
-                    )
+                data["worker_number"] = worker_numbers.get(data.get("stream"))
+                if msg_handler:
+                    msg_handler(data)
                 else:
-                    print(
-                        f"{timestamp.strftime('%m/%d/%Y, %H:%M:%S')}: {data.get('msg').rstrip()}"
+                    timestamp = datetime.fromtimestamp(
+                        int(data.get("time")) / 1000
                     )
+                    if len(self._workers) > 1:
+                        print(
+                            f"{timestamp.strftime('%m/%d/%Y, %H:%M:%S')}: Worker {data.get('worker_number')} - {data.get('msg').rstrip()}"
+                        )
+                    else:
+                        print(
+                            f"{timestamp.strftime('%m/%d/%Y, %H:%M:%S')}: {data.get('msg').rstrip()}"
+                        )
 
-        return msg_handler
+        return handler
 
-    async def attach(self):
+    async def attach(self, msg_handler=None):
         await self.trainml._ws_subscribe(
-            "job", self.id, self._get_msg_handler()
+            "job", self.id, self._get_msg_handler(msg_handler)
         )
 
     async def copy(self, name, **kwargs):
         logging.debug(f"copy request - name: {name} ; kwargs: {kwargs}")
-        if self.type != "interactive":
-            raise TypeError("Only interactive job types can be copied")
+        if self.type not in ["interactive", "notebook"]:
+            raise SpecificationError(
+                "job", "Only notebook job types can be copied"
+            )
 
         job = await self.trainml.jobs.create(
             name,
@@ -308,12 +328,24 @@ class Job:
         return job
 
     async def wait_for(self, status, timeout=300):
-        valid_statuses = ["running", "stopped", "archived"]
+        valid_statuses = ["running", "stopped", "finished", "archived"]
         if not status in valid_statuses:
-            raise ValueError(
-                f"Invalid wait_for status {status}.  Valid statuses are: {valid_statuses}"
+            raise SpecificationError(
+                "status",
+                f"Invalid wait_for status {status}.  Valid statuses are: {valid_statuses}",
             )
-        if self.status == status:
+        if (
+            self.type == "headless" or self.type == "training"
+        ) and status == "stopped":
+            warnings.warn(
+                "'stopped' status is deprecated for training jobs, use 'finished' instead.",
+                DeprecationWarning,
+            )
+        if self.status == status or (
+            (self.type == "headless" or self.type == "training")
+            and status == "finished"
+            and self.status == "stopped"
+        ):
             return
         POLL_INTERVAL = 5
         retry_count = math.ceil(timeout / POLL_INTERVAL)
@@ -326,7 +358,11 @@ class Job:
                 if status == "archived" and e.status == 404:
                     return
                 raise e
-            if self.status == status:
+            if self.status == status or (
+                (self.type == "headless" or self.type == "training")
+                and status == "finished"
+                and self.status == "stopped"
+            ):
                 return self
             elif self.status == "failed":
                 raise JobError(self.status, self)
@@ -334,4 +370,4 @@ class Job:
                 count += 1
                 logging.debug(f"self: {self}, retry count {count}")
 
-        raise TimeoutError(f"Timeout waiting for {status}")
+        raise TrainMLException(f"Timeout waiting for {status}")
